#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/03/07
#Last Modify: 2018/03/07
# Mastering Machine Learning with Python in Six Steps page 172
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import sklearn.linear_model as lm
from sklearn.metrics import r2_score, mean_absolute_error, mean_squared_error

# Load data
df = pd.read_csv('~/test_grades.csv')
#print df

# Simple scatter plot
df.plot(kind='scatter', x='Hours_Studied', y='Test_Grade', title='Grade vs Hours Studied')

# check the correlation between variables
print df.corr()
print

# Create linear regression object
lr = lm.LinearRegression()
x= df.Hours_Studied[:, np.newaxis] # independent variable
y= df.Test_Grade.values # dependent variable

# Train the model using the training sets
lr.fit(x, y)
print "Intercept: ", lr.intercept_
print "Coefficient: ", lr.coef_
print

# predict using the built-in function
print "Using predict(6) function: ", lr.predict(6)

# plotting fitted line
plt.scatter(x, y, color='black')
plt.plot(x, lr.predict(x), color='blue', linewidth=3)
plt.title('Grade vs Hours Studied')
plt.ylabel('Test_Grade')
plt.xlabel('Hours_Studied')

print
# add predict value to the data frame
df['Test_Grade_Pred'] = lr.predict(x)
# Manually calculating R Squared
df['SST'] = np.square(df['Test_Grade'] - df['Test_Grade'].mean())
df['SSR'] = np.square(df['Test_Grade_Pred'] - df['Test_Grade'].mean())
print df
print "Sum of SSR:", df['SSR'].sum()
print "Sum of SST:", df['SST'].sum()
print "R Squared using manual calculation: ", df['SSR'].sum() / df['SST'].sum()
# Using built-in function
print "R Squared using built-in function: ", r2_score(df.Test_Grade, lr.predict(x))
print "Mean Absolute Error: ", mean_absolute_error(df.Test_Grade, df.Test_Grade_Pred)
print "Root Mean Squared Error: ", np.sqrt(mean_squared_error(df.Test_Grade, df.Test_Grade_Pred))

# really manual calculations
#print
#test_grade_predictions = lr.predict(x)
#print "predictions", test_grade_predictions
#sst = (y-79.77)**2
#print "sst = ", sst
#print "sst.sum() = ",sum(sst)
#ssr = (test_grade_predictions-79.77)**2
#print "ssr = ", ssr
#print "ssr.sum() = ",sum(ssr)
#ssr = (test_grade_predictions-79.77)**2
#r_squared = sum(ssr) / sum(sst)
#print "r_squared = ", r_squared
#plt.show()
